<?xml version="1.0"?>

<!-- $Id$ -->

<!--
     SIMP Doc XSL Transformer

     This transformer adds automatic section numbering to an XHTML 1.0
     Basic document, and then generates an XHTML 1.0 Strict docmument.

     John D. Ramsdell - The MITRE Corporation

     Copyright (c) 2002. The MITRE Corporation (http://www.mitre.org/).
     All rights reserved.
     The SIMP Service comes with ABSOLUTELY NO WARRANTY.
     See the SIMP Service License Agreement for details.

     Version History

     * 2002-Apr-29

       initial version

     * 2002-Apr-30

       Allow sections to generate h4, h5, and h6 elements.  
       Added class indicator for sections and references.
       Added nbsp enity.
       Dumped support for abstracts.
-->

<!DOCTYPE xsl:stylesheet [
  <!ENTITY nbsp "&#0160;">
]>

<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns="http://www.w3.org/1999/xhtml"
    xmlns:sd='http://simp.mitre.org/simpdoc'
    exclude-result-prefixes="sd"
    version="1.0">

<!-- the table of contents boolean parameter -->

<!-- get toc attribute from the simpdoc processing instruction -->

<xsl:variable name="pi-toc"
    select="substring-after(
        /processing-instruction('simpdoc')[contains(.,'toc=')],
        'toc=')"/>

<!-- true when table of contents should be generated -->

<xsl:param name="make-table-of-contents"
    select="not($pi-toc)
            or contains($pi-toc, 'yes')
            or contains($pi-toc, 'true')"/>

<!-- Currently, this style sheet generates an XHTML 1.0 compliant
     document.  It includes a system reference to a DTD that is a URL.
     You may want to changes this to be a local file reference.

     "TR/xhtml1/DTD/xhtml1-strict.dtd" -->

<xsl:output
    method='xml'
    doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
    doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"
    encoding='iso-8859-1'
    media-type='text/html'
    version='1.0' />

<!-- The following comment comes from the XHTML 1.0 Strict DTD. -->

<!--
   Extensible HTML version 1.0 Strict DTD

   This is the same as HTML 4.0 Strict except for
   changes due to the differences between XML and SGML.

   Namespace = http://www.w3.org/1999/xhtml

   For further information, see: http://www.w3.org/TR/xhtml1

   Copyright (c) 1998-2000 W3C (MIT, INRIA, Keio),
   All Rights Reserved.

   This DTD module is identified by the PUBLIC and SYSTEM identifiers:

   PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
   SYSTEM "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"

   $Revision: 1.14 $
   $Date: 2000/01/25 23:52:20 $

-->

<!-- Someday, this should produce XHTML Basic DTD.  This requires
     changes to the anchor generators, because "name" is no longer a
     recognized attribute.  -->

<!-- The following comment comes from the XHTML Basic DTD. -->

<!-- This is the driver file for version 1.0 of the XHTML Basic DTD.

     This DTD is identified by the PUBLIC and SYSTEM identifiers:

     PUBLIC: "-//W3C//DTD XHTML Basic 1.0//EN"
     SYSTEM: "http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd"
-->

<!--
<xsl:output
    method='xml'
    doctype-public="-//W3C//DTD XHTML Basic 1.0//EN"
    doctype-system="http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd"
    encoding='iso-8859-1'
    media-type='text/html'
    version='1.0' />
-->

<!-- root template -->

<xsl:template match='/'>
  <xsl:apply-templates />
</xsl:template>

<!-- produce document structure -->

<xsl:template match="sd:simpdoc">
  <html xml:lang="en">
    <head>
      <title><xsl:value-of select="sd:front/sd:title/text()" /></title>
      <xsl:apply-templates mode="head"/>
    </head>
    <body>
      <xsl:apply-templates />
    </body>
  </html>
</xsl:template>

<!-- head content -->

<xsl:template match="sd:head" mode="head">
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="node()" mode="head"/>

<xsl:template match="sd:head"/>

<!-- table of contents -->

<xsl:template match="sd:section|sd:references|sd:appendix" mode="toc">
  <tr>
    <td valign="top">
      <xsl:apply-templates select="." mode="get-number"/>
      <xsl:text>&nbsp;</xsl:text>
    </td>
    <td>
      <a>
        <xsl:attribute name="href">
          <xsl:text>#</xsl:text>
          <xsl:apply-templates select="." mode="get-anchor"/>
        </xsl:attribute>
        <xsl:value-of select="@title"/>
      </a>
    </td>
  </tr>
  <xsl:apply-templates mode="toc"/>
</xsl:template>

<xsl:template match="node()" mode="toc"/>

<!-- numbering -->

<xsl:template match="sd:section|sd:references" mode="get-number">
  <xsl:number count="sd:section|sd:references"
      level="multiple" format="1"/>
</xsl:template>

<xsl:template match="sd:appendix|sd:appendix//sd:section" mode="get-number">
  <xsl:number count="sd:appendix|sd:appendix//sd:section"
      level="multiple" format="A.1" />
</xsl:template>

<xsl:template match="sd:reference" mode="get-number">
  <xsl:apply-templates select="parent::sd:references" mode="get-number"/>
  <xsl:text>.</xsl:text>
  <xsl:number count="sd:reference" level="single" format="1"/>
</xsl:template>

<xsl:template match="sd:figure" mode="get-number">
  <xsl:number count="sd:figure" level="any" format="1"/>
</xsl:template>

<xsl:template match="*" mode="get-number"/>

<!-- anchors -->

<xsl:template
    match="sd:section|sd:references|sd:reference|sd:appendix|sd:figure"
    mode="get-anchor">
  <xsl:choose>
    <xsl:when test="@id">
      <xsl:value-of select="@id" />
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="local-name()" />
      <xsl:text>.</xsl:text>
      <xsl:apply-templates select="." mode="get-number"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="*" mode="get-anchor"/>

<!-- sections and appendixes -->

<xsl:template match="sd:section|sd:appendix">
  <!-- choose header element -->
  <xsl:variable name="depth"
       select="count(ancestor::sd:section) + count(ancestor::sd:appendix)"/>
  <xsl:variable name="elemtype">
    <xsl:choose>
      <xsl:when test="$depth > 4">
        <xsl:text>h6</xsl:text>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="concat('h', $depth + 1)"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <!-- emit header element with an anchor -->
  <xsl:element name="{$elemtype}">
    <xsl:attribute name="class">section</xsl:attribute>
    <a>
      <xsl:attribute name="id">
        <xsl:apply-templates select="." mode="get-anchor"/>
      </xsl:attribute>
      <!-- for compatibility with old HTML standards -->
      <xsl:attribute name="name">
        <xsl:apply-templates select="." mode="get-anchor"/>
      </xsl:attribute>
      <xsl:apply-templates select="." mode="get-number"/>
      <xsl:text>.&nbsp; </xsl:text>
      <xsl:value-of select="@title" />
    </a>
  </xsl:element>
  <xsl:apply-templates />
</xsl:template>

<xsl:template match="sd:section" mode="name-anchor">
  <xsl:text>Section </xsl:text>
  <xsl:apply-templates select="." mode="get-number"/>
</xsl:template>

<xsl:template match="sd:appendix" mode="name-anchor">
  <xsl:text>Appendix </xsl:text>
  <xsl:apply-templates select="." mode="get-number"/>
</xsl:template>

<!-- figures -->

<!-- This version fails to emit an anchor with a name attribute when
     the figure has no title.  -->

<xsl:template match="sd:figure">
  <blockquote class="figure">
    <xsl:attribute name="id">
      <xsl:apply-templates select="." mode="get-anchor"/>
    </xsl:attribute>
    <xsl:apply-templates />
    <xsl:if test="@title">
      <p>
        <a>
          <!-- for compatibility with old HTML standards -->
          <xsl:attribute name="name">
            <xsl:apply-templates select="." mode="get-anchor"/>
          </xsl:attribute>
          <xsl:apply-templates select="." mode="name-anchor"/>
          <xsl:text>.&nbsp; </xsl:text>
          <xsl:value-of select="@title"/>
        </a>
      </p>
    </xsl:if>
  </blockquote>
</xsl:template>

<xsl:template match="sd:figure" mode="name-anchor">
  <xsl:text>Figure </xsl:text>
  <xsl:number count="sd:figure[@title or @id]" level="any" format="1"/>
</xsl:template>

<!-- cross references -->

<xsl:template match="sd:xref">
  <xsl:variable name="target" select="@target" />
  <xsl:variable name="node" select="//*[@id=$target]" />
  <xsl:choose>
    <xsl:when test="$node">
      <xsl:text>[</xsl:text>
      <a>
        <xsl:attribute name="href">
          <xsl:text>#</xsl:text>
          <xsl:value-of select="$target"/>
        </xsl:attribute>
        <xsl:apply-templates select="$node" mode="name-anchor"/>
      </a>
      <xsl:if test="node()">
        <xsl:text>, </xsl:text>
        <xsl:apply-templates/>
      </xsl:if>
      <xsl:text>]</xsl:text>
    </xsl:when>
    <xsl:otherwise>
      <xsl:message>
        <xsl:text>Anchor </xsl:text>
        <xsl:value-of select="$target"/>
        <xsl:text> missing.</xsl:text>
      </xsl:message>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- collections of references -->

<xsl:template match="sd:references">
  <h1 class="references">
    <a>
      <xsl:attribute name="id">
        <xsl:apply-templates select="." mode="get-anchor"/>
      </xsl:attribute>
      <!-- for compatibility with old HTML standards -->
      <xsl:attribute name="name">
        <xsl:apply-templates select="." mode="get-anchor"/>
      </xsl:attribute>
      <xsl:apply-templates select="." mode="get-number"/>
      <xsl:text>.&nbsp; </xsl:text>
      <xsl:value-of select="@title" />
    </a>
  </h1>
  <dl class="references">
    <xsl:apply-templates />
  </dl>
</xsl:template>

<xsl:template match="sd:references" mode="name-anchor">
  <xsl:text>References </xsl:text>
  <xsl:apply-templates select="." mode="get-number"/>
</xsl:template>

<!-- a reference entry -->

<xsl:template match="sd:reference">
  <dt>
    <xsl:text>[</xsl:text>
    <a>
      <xsl:attribute name="id">
        <xsl:apply-templates select="." mode="get-anchor"/>
      </xsl:attribute>
      <!-- for compatibility with old HTML standards -->
      <xsl:attribute name="name">
        <xsl:apply-templates select="." mode="get-anchor"/>
      </xsl:attribute>
      <xsl:if test="@target">
        <xsl:attribute name="href">
          <xsl:value-of select="@target"/>
        </xsl:attribute>
      </xsl:if>
      <xsl:apply-templates select="." mode="name-anchor"/>
    </a>
    <xsl:text>]</xsl:text>
  </dt>
  <dd>
    <xsl:apply-templates />
  </dd>
</xsl:template>

<xsl:template match="sd:reference" mode="name-anchor">
  <xsl:choose>
    <xsl:when test="@id">
      <xsl:value-of select="@id" />
    </xsl:when>
    <xsl:otherwise>
      <xsl:apply-templates select="." mode="get-number"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- front matter -->

<xsl:template match="sd:front">
  <xsl:apply-templates />
  <xsl:if test="$make-table-of-contents">
    <h2 class="toc">Table of Contents</h2>
    <table class="toc">
      <xsl:apply-templates select="/sd:simpdoc/*" mode="toc"/>
    </table>
  </xsl:if>
</xsl:template>

<xsl:template match="sd:title">
  <h1 class="title">
    <xsl:apply-templates />
  </h1>
</xsl:template>

<!-- some default templates -->

<xsl:template match="*" mode="name-anchor"/>

<!-- copy from source to result tree -->

<xsl:template match="*">
  <xsl:element name="{name()}">
    <xsl:copy-of select="@*"/>
    <xsl:apply-templates />
  </xsl:element>
</xsl:template>

<!-- Note that if you use xsl:copy instead of xsl:element, you run
     into namespace problems.  The following template produces the
     wrong result.

<xsl:template match="*|@*">
  <xsl:copy>
    <xsl:apply-templates select="@*"/>
    <xsl:apply-templates />
  </xsl:copy>
</xsl:template>

-->

</xsl:stylesheet>
